@@ -1,6 +1,6 @@
 # Module for XML, HTML and CGI output
 # jaxml
-# (C) Jerome Alet <alet@unice.fr> 2000-2001
+# (C) Jerome Alet <alet@librelogiciel.com> 2000-2002
 # You're welcome to redistribute this software under the
 # terms of the GNU General Public Licence version 2.0
 # or, at your option, any higher version.
@@ -9,9 +9,23 @@
 # which should come along with this software, or visit
 # the Free Software Foundation's WEB site http://www.fsf.org
 #
-# $Id: jaxml.py,v 1.39 2002/03/02 09:19:36 jerome Exp $
+# $Id: jaxml.py,v 1.43 2003/06/26 06:59:32 jerome Exp $
 #
 # $Log: jaxml.py,v $
+# Revision 1.43  2003/06/26 06:59:32  jerome
+# Small fix.
+#
+# Revision 1.42  2003/02/13 14:36:09  jerome
+# Version 3.0
+#
+# Revision 1.41  2003/02/13 10:33:58  jerome
+# Version number changed to 3.0beta
+# Named _push() and _pop() possibility (untested)
+# Complete namespaces support thanks to Jean Jordaan
+#
+# Revision 1.40  2002/04/25 09:08:34  jerome
+# New copyright strings
+#
 # Revision 1.39  2002/03/02 09:19:36  jerome
 # typo in _do_nothing() in CGI scripts
 #
@@ -210,11 +224,12 @@
 #
 #
 import sys
+import os
 import string
 import cStringIO
 import time
 
-__version__ = "2.23"
+__version__ = "3.02"
 __doc__ = """
 This python module defines a class named XML_document which will
 allow you to generate XML documents (yeah !) more easily than
@@ -250,12 +265,17 @@ Here's a list of available methods:
                 use it if you're going to create a bunch of nested
                 XML tags and want to escape from them later to continue
                 your document at the same indentation level.
+                you can pass an optional 'name' argument, to mark
+                a position by its name.
 
         _pop()
                 restores the latest saved position.
                 use it to escape from nested tags and continue
                 your XML document at the same indentation level than
                 the latest time you called _push().
+                you can pass an optional 'name' argument, to continue
+                at the same indentation level as when you called _push()
+                with the same 'name' argument.
 
         _template(file, **vars)
                 loads a template file and insert it as plain text at the current
@@ -288,6 +308,8 @@ attempts at describing and trying all jaxml's possibilities, so reading
 it is probably the best way to become powerful with jaxml in less than
 10 minutes.
 
+Really, PLEASE READ the file test/test.py to learn all possibilities.
+
 =========================================================================
 
 Since version 2.00, jaxml integrates the full functionnalities of the
@@ -311,7 +333,14 @@ The only difficult things are:
 
         * if you call a method (tag) with a string as the first
           unnamed parameter, you'll don't need _push() or _pop()
-          because your tag will be automatically closed immediately."""
+          because your tag will be automatically closed immediately.
+          
+        * if you call a method (tag) with a python mapping as the
+          first or second unamed parameter, this mapping is used
+          to correctly handle XML namespaces or attributes
+          which are python reserved words (e.g. class), please
+          look at test/test.py to see an example.
+"""
 
 class _TAGGED_document :
         """This class defines a tagged document"""
@@ -334,7 +363,7 @@ class _TAGGED_document :
                         self.__parent = parent
                         self.__tagname = tagname
 
-                def __call__(self, _text_ = None, **attributes) :
+                def __call__(self, _text_ = None, *nsattributes, **attributes) :
                         """Inserts the tag and its attributes in the document
 
                            _text_
@@ -342,6 +371,28 @@ class _TAGGED_document :
                                 name _text_ was chosen to not conflict with a probable user's attribute
                                 called 'text'
                         """
+                        #
+                        # NameSpace idea from Jean Jordaan
+                        if type(_text_) == type({}) :
+                                nsattributes = (_text_, )
+                                _text_ = None
+                        nsargs = ""
+                        lg = len(nsattributes)
+                        if (lg > 1) :
+                                raise ValueError, "jaxml: Invalid attributes %s" % str(nsattributes[0])
+                        elif lg :    
+                                nsattr = nsattributes[0]
+                                try :
+                                        for ns in nsattr.keys() :
+                                                tags = nsattr[ns]
+                                                try :
+                                                        for tag in tags.keys() :        
+                                                                nsargs = nsargs + ' %s%s%s="%s"' % (ns, (ns and ':'), tag, str(tags[tag]))
+                                                except AttributeError :                
+                                                        nsargs = nsargs + ' %s="%s"' % (ns, str(tags))
+                                except AttributeError :                        
+                                        raise ValueError, "jaxml: Invalid attributes %s" % str(nsattr)
+                                        
                         # first, we compute the attributes string
                         # we vonluntarily do the test because of the speed optimisation
                         # it gives when there's no attribute
@@ -355,10 +406,10 @@ class _TAGGED_document :
                         # if a "first" argument was passed, enclose it in the tag
                         # and just get out of this tag
                         if _text_ is not None :
-                                self.__parent._text("<%s%s>%s</%s>" % (self.__tagname, arg, str(_text_), self.__tagname))
+                                self.__parent._text("<%s%s>%s</%s>" % (self.__tagname, arg + nsargs, str(_text_), self.__tagname))
                         else :
                                 # future tags will be inserted inside this one
-                                self.__parent._tag__(self.__tagname, arg)
+                                self.__parent._tag__(self.__tagname, arg + nsargs)
                         return self.__parent
 
                 def __getattr__(self, name) :
@@ -377,6 +428,7 @@ class _TAGGED_document :
                 # the document itself
                 self.__page = []
                 self.__pushed = []
+                self.__pusheddict = {}
                 self.__position = 0
 
                 # Initialise a mapping to implement another templating
@@ -394,6 +446,7 @@ class _TAGGED_document :
                 # copy the "private" members
                 new.__page = self.__page[:]
                 new.__pushed = self.__pushed[:]
+                new.__pusheddict = self.__pusheddict.copy()
                 new.__position = self.__position
                 new.__indentstring = self.__indentstring
                 new.__mapping = self.__mapping.copy()
@@ -579,8 +632,8 @@ class _TAGGED_document :
                         offset by which adjust the stack
                 """
                 if self.__pushed :
-                        pos, oldoffset = self.__pushed[-1]
-                        self.__pushed[-1] = (pos, oldoffset + offset)
+                        pos, oldoffset = self.__pushed.pop()
+                        self.__pushed.append((pos, oldoffset + offset))
 
         def _tag__(self, tag, arg) :
                 self.__page.insert(self.__position, (tag, arg, 1))
@@ -590,22 +643,39 @@ class _TAGGED_document :
 
                 #
                 # Callable interface starts here
-        def _push(self) :
+        def _push(self, name=None) :
                 """Push the current tag's position.
 
                    useful before a block of nested tags
+                   
+                   name : can be used to name the pushed position and pop it later directly
                 """
+                if name :
+                        self.__pusheddict[name] = len(self.__pushed)
                 self.__pushed.append((self.__position, 0))
 
-        def _pop(self) :
+        def _pop(self, name=None) :
                 """Restore the latest pushed position.
 
                    useful to get out of a block of nested tags
+                   
+                   name : can be used to restore a named position, not necessarily the latest.
                 """
                 if self.__pushed :
-                        pos, offset = self.__pushed.pop()
-                        self.__position = pos + offset
-                        self.__adjust_stack(offset) # we report the offset on previously saved tags
+                        maxindex = len(self.__pushed) - 1
+                        if name :
+                                try :
+                                        index = self.__pusheddict[name]
+                                        del self.__pusheddict[name]
+                                except KeyError :        
+                                        raise KeyError, "jaxml named position %s doesn't exist" % name
+                        else :
+                                index = maxindex
+                        while maxindex >= index :
+                                pos, offset = self.__pushed.pop()
+                                self.__position = pos + offset
+                                self.__adjust_stack(offset) # we report the offset on previously saved tags
+                                maxindex = maxindex - 1
 
         def _text(self, text):
                 """Insert plain text in the document
@@ -917,7 +987,7 @@ class CGI_document(HTML_document) :
 
     def _set_debug(self, file) :
         """Sets the flag to send the output to a file too."""
-        self.__debug__ = file
+        self.__debug_file__ = file
 
     def _set_pragma(self, pragma) :
         """Defines the pragma value.
@@ -1075,12 +1145,12 @@ class CGI_document(HTML_document) :
     def _output(self, file = "-") :
         """Prints the CGI script output to stdout or file.
 
-           If self.__debug__ is defined it is used as a file
+           If self.__debug_file__ is defined it is used as a file
            to which send the output to too.
         """
         HTML_document._output(self, file)
-        if self.__debug__ :
-                HTML_document._output(self, self.__debug__)
+        if self.__debug_file__ :
+                HTML_document._output(self, self.__debug_file__)
 
 class Html_document :
         """This class warns the programmer when used, and exits the program.
